<?php

namespace Service;

class Consumer {
    
    protected $connection;
    protected $util;
    protected $localUtil;
    
    public function __construct($connection, \Odhen\API\Util\Util $util, \Util\Util $localUtil) {
        $this->connection = $connection;
        $this->util = $util;
        $this->localUtil = $localUtil;
    }
    
    public function getConsumersByPeriod($DTINIPERIODO, $DTFIMPERIODO) {
        $params = array(
            ':DTINIPERIODO' => $DTINIPERIODO,
            ':DTFIMPERIODO' => $DTFIMPERIODO
        );
        try {
            // verifica qual banco de dados está sendo utilizado
            if ($this->util->databaseIsOracle()) {
                $consumers = $this->connection->fetchAll(\Util\Query::ORACLE_GET_CONSUMERS_BY_PERIOD, $params);
                
            } else {
                $consumers = $this->connection->fetchAll(\Util\Query::SQL_GET_CONSUMERS_BY_PERIOD, $params);
            }
            $return = array(
                'error' => false,
                'data'  => $consumers
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar consumidores. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }
    
    public function getConsumerAditionalAddresses($CDCLIENTE, $CDCONSUMIDOR) {
        $params = array(
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        
        try {
            $aditionalAddresses = $this->connection->fetchAll(\Util\Query::GET_CONSUMERS_ADITIONAL_ADDRESSES, $params);
            $return = array(
                'error' => false,
                'data'  => $aditionalAddresses
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar endereços adicionais do consumidor '.$CDCONSUMIDOR.'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    public function saveConsumer($consumer) {
        $connection = $this->connection;
        
        if($this->util->databaseIsOracle()) {
            if(empty($consumer['CDCONSUMIDOR'])) {
                $consumer['CDCONSUMIDOR'] = "0".$this->localUtil->newCode("CONSUMIDOR", 20);
            }

            $idatuconsumi = 'S';
            
        } else {
            if(empty($consumer['CDCONSUMIDOR'])) {
                $consumer['CDCONSUMIDOR'] = "0".$this->util->geraCodigo($connection, 'CONSUMIDOR', '1', 1, 20);
            }
            
            $idatuconsumi = 'I';
        }
        
        $params = array(
            'CDCLIENTE' => $consumer['CDCLIENTE'],
            'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR'],
            'CDIDCONSUMID' => $consumer['CDIDCONSUMID'],
            'CDTIPOCONS' => $consumer['CDTIPOCONS'],
            'CDCCUSCLIE' => $consumer['CDCCUSCLIE'],
            'NRCPFRESPCON' => $consumer['NRCPFRESPCON'],
            'NMCONSUMIDOR' => $consumer['NMCONSUMIDOR'],
            'DSEMAILCONS' => $consumer['DSEMAILCONS'],
            'DTNASCCONS' => $consumer['DTNASCCONS'],
            'IDATIVO' => $consumer['IDATIVO'],
            'IDSEXOCONS' => $consumer['IDSEXOCONS'],
            'NRCEPCONS' => $consumer['NRCEPCONS'],
            'DSENDECONS' => $consumer['DSENDECONS'],
            'NRENDECONS' => $consumer['NRENDECONS'],
            'CDMUNICIPIO' => $consumer['CDMUNICIPIO'],
            'SGESTADO' => $consumer['SGESTADO'],
            'CDPAIS' => $consumer['CDPAIS'],
            'DSCOMPLENDECONS' => $consumer['DSCOMPLENDECONS'],
            'NMBAIRCONS' => $consumer['DSBAIRRO'],
            'NRTELECONS' => $consumer['NRTELECONS'],
            'NRCELULARCONS' => $consumer['NRCELULARCONS'],
            'IDATUCONSUMI' => $idatuconsumi
        );
        
        try {
            if ($this->util->databaseIsOracle()) {
                $success = $this->connection->executeQuery(\Util\Query::ORACLE_INSERT_CONSUMER, $params);
            } else {
                $success = $this->connection->executeQuery(\Util\Query::SQL_INSERT_CONSUMER, $params);
            }
            $return = array(
                'error' => false,
                'data' => array('consumerCode' => $consumer['CDCONSUMIDOR'])
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao inserir dados do consumidor '.$consumer['CDCONSUMIDOR'].'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function addAdditionalAddress($address) {
        $params = array(
            'CDCLIENTE'       => $address['CDCLIENTE'],
            'CDCONSUMIDOR'    => $address['CDCONSUMIDOR'],
            'CDENDECONS'      => $address['CDENDECONS'],
            'NOMEENDE'        => $address['NOMEENDE'],
            'NRCEPCONS'       => $address['NRCEPCONS'],
            'DSENDECONS'      => $address['DSENDECONS'],
            'NRENDECONS'      => $address['NRENDECONS'],
            'CDMUNICIPIO'     => $address['CDMUNICIPIO'],
            'SGESTADO'        => $address['SGESTADO'],
            'CDPAIS'          => $address['CDPAIS'],
            'DSCOMPLENDECONS' => $address['DSCOMPLENDECONS'],
            'DSBAIRRO'        => $address['DSBAIRRO'],
        );
        
        try {
            $success = $this->connection->executeQuery(\Util\Query::INSERT_ADITIONAL_ADDRESS, $params);
            $return = array(
                'error' => false
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao inserir endereço adicional "'.$address['NOMEENDE'].'" para o consumidor '.$address['CDCONSUMIDOR'].'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    public function getSalesByPeriod($CDCLIENTE, $CDCONSUMIDOR, $DTINIPERIODO, $DTFIMPERIODO, $COUNTER, $CHECKDATE, $CHECKCOUNTER, $subsidiaries) {
        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'DTINIPERIODO' => $DTINIPERIODO,
            'DTFIMPERIODO' => $DTFIMPERIODO,
            'COUNTER' => $COUNTER,
            'CHECKDATE' => $CHECKDATE,
            'CHECKCOUNTER' => $CHECKCOUNTER,
            'CDFILIAL' => $subsidiaries
        );
        $paramsType = array('CDFILIAL' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY);
        try {
            if ($this->util->databaseIsOracle()) $sales = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SALES_BY_PERIOD, $params, $paramsType);
            else  $sales = $this->connection->fetchAll(\Util\Query::SQL_GET_SALES_BY_PERIOD, $params, $paramsType);
            $return = array(
                'error' => false,
                'data' => $sales
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar vendas. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function getLastSale($NRCOMANDA, $CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR) {
        $params = array(
            'NRCOMANDA' => $NRCOMANDA,
            'CDFILIAL' => $CDFILIAL,
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        $errorMessage = 'Erro ao buscar a venda.';
        try {
            $isComandaVen = true;
            if ($this->util->databaseIsOracle()) {
                $result = $this->connection->fetchAssoc(\Util\Query::ORACLE_GET_LAST_SALE, $params);
                if(empty($result)){
                    $result = $this->connection->fetchAssoc(\Util\Query::ORACLE_GET_LAST_SALE_VENDA, $params);
                    $isComandaVen = false;
                }
                
            } else $result = $this->connection->fetchAssoc(\Util\Query::SQL_GET_LAST_SALE, $params);

            if($result) {
                $return = array(
                    'error' => false,
                    'data' => $result,
                    'isComandaVen' => $isComandaVen
                );
            } else {
                $return = array(
                    'error' => true,
                    'message' => $errorMessage
                );
            }
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => $errorMessage.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getLastSaleProducts($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDCLIENTE, $CDCONSUMIDOR, $isComandaVen) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        try {
            if ($this->util->databaseIsOracle()) {
                $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_LASTSALE_PRODUCTS, $params);
                if(empty($products) && !$isComandaVen) $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_LASTSALE_PRODUCTS_ITEMVENDA, $params);
            } else $products = $this->connection->fetchAll(\Util\Query::SQL_GET_LASTSALE_PRODUCTS, $params);
            $return = array(
                'error' => false,
                'data' => $products
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos relacionados à venda '.$NRCOMANDA.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getLastSaleCombos($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDCLIENTE, $CDCONSUMIDOR, $isComandaVen) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'CDCLIENTE' => $CDCLIENTE,   
            'CDCONSUMIDOR' => $CDCONSUMIDOR    
        );
        try {
            $combos = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_COMBOS, $params);
            if(empty($combos) && !$isComandaVen) $combos = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_COMBOS_ITEMVENDA, $params);
            $return = array(
                'error' => false,
                'data' => $combos
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar combos relacionados à venda. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    public function getLastSaleComboProducts($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDPRODPROMOCAO, $NRSEQPRODCOM, $CDCLIENTE,$CDCONSUMIDOR,$isComandaVen) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'CDPRODPROMOCAO' => $CDPRODPROMOCAO,
            'NRSEQPRODCOM' => $NRSEQPRODCOM,
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        try {
            if ($this->util->databaseIsOracle()) {
                $comboProducts = $this->connection->fetchAll(\Util\Query::ORACLE_GET_LASTSALE_COMBO_PRODUCTS, $params);
                if(empty($comboProducts) && !$isComandaVen) $comboProducts = $this->connection->fetchAll(\Util\Query::ORACLE_GET_LASTSALE_COMBO_PRODUCTS_ITEMVENDA, $params);
            } else $comboProducts = $this->connection->fetchAll(\Util\Query::SQL_GET_POSITION_COMBO_PRODUCTS, $params);
            $return = array(
                'error' => false,
                'data' => $comboProducts
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos dos combos relacionados à venda'.$e->getMessage()
            );
        }
        return $return;
    }

    public function getLastSalePayments($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDCLIENTE, $CDCONSUMIDOR, $isComandaVen) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST,
            'NRCOMANDA' => $NRCOMANDA,
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        try {
            $payments = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PAYMENTS, $params);
            if(empty($payments) && !$isComandaVen)$payments = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PAYMENTS_MOVCAIXA, $params);
            $return = array(
                'error' => false,
                'data' => $payments
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar formas de pagamento relacionadas à venda '.$NRCOMANDA.'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    public function getLastSaleProductObservations($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN, $isComandaVen){
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'NRPRODCOMVEN' => $NRPRODCOMVEN
        );
        try {
            if ($this->util->databaseIsOracle()) {
                $observations = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PRODUCT_OBSERVATIONS, $params);
                if(empty($observations) && !$isComandaVen) $observations = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PRODUCT_OBSERVATIONS_OBSITEMVENDA, $params);
            } else $observations = $this->connection->fetchAll(\Util\Query::GET_POSITION_PRODUCT_OBSERVATIONS, $params);
            $return = array(
                'error' => false,
                'data' => $observations
            );

        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar observacoes. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getSaleProductObservations($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA,
            'NRSEQUITVEND' => $NRSEQUITVEND    
        );
        try {
            if ($this->util->databaseIsOracle()) $observations = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SALE_PRODUCT_OBSERVATIONS, $params);
            else $observations = $this->connection->fetchAll(\Util\Query::SQL_GET_SALE_PRODUCT_OBSERVATIONS, $params);
      
            $return = array(
                'error' => false,
                'data' => $observations
            );

        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar observacoes. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    public function getProducts($CDFILIAL, $CDCAIXA, $NRSEQVENDA) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'CDCAIXA' => $CDCAIXA, 
            'NRSEQVENDA' => $NRSEQVENDA    
        );
        
        try {
            if ($this->util->databaseIsOracle()) $products = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SALE_PRODUCTS, $params);
            else $products = $this->connection->fetchAll(\Util\Query::SQL_GET_SALE_PRODUCTS, $params);
            $return = array(
                'error' => false,
                'data' => $products
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos relacionados à venda '.$NRSEQVENDA.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function getCombos($CDFILIAL, $CDCAIXA, $NRSEQVENDA) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'CDCAIXA' => $CDCAIXA, 
            'NRSEQVENDA' => $NRSEQVENDA
        );
        try {
            if ($this->util->databaseIsOracle()) $combos = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SALE_COMBOS, $params);
            else $combos = $this->connection->fetchAll(\Util\Query::SQL_GET_SALE_COMBOS, $params);
            $return = array(
                'error' => false,
                'data' => $combos
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar combos relacionados à venda '.$NRSEQVENDA.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function getComboProducts($CDFILIAL, $CDCAIXA, $NRSEQVENDA,$NRSEQUITVEND,$NRSEQPRODCOMIT,$AGRUPADO) {
        try {
            if ($AGRUPADO == 'S') {
                $params = array(
                    'CDFILIAL' => $CDFILIAL, 
                    'CDCAIXA' => $CDCAIXA, 
                    'NRSEQVENDA' => $NRSEQVENDA,
                    'NRSEQUITVEND' => $NRSEQUITVEND   
                );
                $comboProducts = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SALE_GROUPED_COMBO_PRODUCTS, $params);
            } else {
                $params = array(
                    'CDFILIAL' => $CDFILIAL, 
                    'CDCAIXA' => $CDCAIXA, 
                    'NRSEQVENDA' => $NRSEQVENDA,
                    'NRSEQUITVEND' => $NRSEQUITVEND,
                    'NRSEQPRODCOMIT' => $NRSEQPRODCOMIT   
                );
                $comboProducts = $this->connection->fetchAll(\Util\Query::ORACLE_GET_SALE_UNGROUPED_COMBO_PRODUCTS, $params);
            }
            $return = array(
                'error' => false,
                'data' => $comboProducts
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos dos combos relacionados à venda '.$NRSEQVENDA.'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    public function getSalePayments($CDFILIAL, $CDCAIXA, $NRSEQVENDA) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'CDCAIXA' => $CDCAIXA, 
            'NRSEQVENDA' => $NRSEQVENDA    
        );
        
        try {
            $payments = $this->connection->fetchAll(\Util\Query::GET_SALE_PAYMENTS, $params);
            $return = array(
                'error' => false,
                'data' => $payments
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar formas de pagamento relacionadas à venda '.$NRSEQVENDA.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }
    
    public function getSalesExtractByPeriod($CDCLIENTE, $CDCONSUMIDOR, $DTINIPERIODO, $DTFIMPERIODO) {
        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'DTINIPERIODO' => $DTINIPERIODO,
            'DTFIMPERIODO' => $DTFIMPERIODO
        );
        
        try {
            if ($this->util->databaseIsOracle()) $extract = $this->connection->fetchAll(\Util\Query::ORACLE_GET_EXTRACT_BY_PERIOD, $params);
            else $extract = $this->connection->fetchAll(\Util\Query::SQL_GET_EXTRACT_BY_PERIOD, $params);
            
            $return = array(
                'error' => false,
                'data' => $extract
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar extrato do consumidor '.$CDCONSUMIDOR.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function editConsumer($consumer) {
        $params = array(
            'CDCLIENTE' => $consumer['CDCLIENTE'],
            'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR'],
            'CDTIPOCONS' => $consumer['CDTIPOCONS'],
            'CDCCUSCLIE' => $consumer['CDCCUSCLIE'],
            'NRCPFRESPCON' => $consumer['NRCPFRESPCON'],
            'NMCONSUMIDOR' => $consumer['NMCONSUMIDOR'],
            'DSEMAILCONS' => $consumer['DSEMAILCONS'],
            'DTNASCCONS' => $consumer['DTNASCCONS'],
            'IDATIVO' => $consumer['IDATIVO'],
            'IDSEXOCONS' => $consumer['IDSEXOCONS'],
            'NRCEPCONS' => $consumer['NRCEPCONS'],
            'DSENDECONS' => $consumer['DSENDECONS'],
            'NRENDECONS' => $consumer['NRENDECONS'],
            'CDMUNICIPIO' => $consumer['CDMUNICIPIO'],
            'SGESTADO' => $consumer['SGESTADO'],
            'CDPAIS' => $consumer['CDPAIS'],
            'DSCOMPLENDECONS' => $consumer['DSCOMPLENDECONS'],
            'DSBAIRRO' => $consumer['DSBAIRRO'],
            'NRTELECONS' => $consumer['NRTELECONS'],
            'NRCELULARCONS' => $consumer['NRCELULARCONS']
        );
        
        try {
            $success = $this->connection->executeQuery(\Util\Query::ORACLE_UPDATE_CONSUMER, $params);
            
            $return = array(
                'error' => false
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao atualizar dados do consumidor '.$consumer['CDCONSUMIDOR'].'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    public function hasOpenSale($CDCONSUMIDOR, $CDCLIENTE, $subsidiaries){
        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'CDFILIAL' => $subsidiaries
        );
        $paramsType = array(
            'CDFILIAL' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
        );
        
        try {
            $openSales = $this->connection->fetchAll(\Util\Query::ORACLE_GET_OPEN_SALES, $params, $paramsType);
            $return = array(
                'error' => false,
                'data'  => $openSales
            );
        } catch(\Exception $e){
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar vendas abertas. Erro: '.$e->getMessage()
            );
        }   
        return $return;
    }

    public function getConsumerById($CDCONSUMIDOR){
        $params = array(
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        
        try {
            $consumer = $this->connection->fetchAssoc(\Util\Query::GET_CONSUMER_BY_ID, $params);
            if($consumer){
                $return = array(
                    'error' => false,
                    'data' => $consumer
                );
            }else{
                $return = array(
                    'error' => false,
                    'message' => 'Consumidor inexistente',
                    'data' => array()
                );
            }    
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar consumidores. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    public function getConsumerByCpf($NRCPFRESPCON) {
        $params = array(
            'NRCPFRESPCON' => $NRCPFRESPCON
        );
        
        try {
            $consumer = $this->connection->fetchAssoc(\Util\Query::GET_CONSUMER_BY_CPF, $params);
            if($consumer){
                $return = array(
                    'error' => false,
                    'data' => $consumer
                );
            }else{
                $return = array(
                    'error' => false,
                    'message' => 'Consumidor inexistente',
                    'data' => array()
                );
            }    
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar consumidores. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getConsumerGroups($CDCONSUMIDOR){
        $params = array(
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        
        try {
            $consumerGroups = $this->connection->fetchAll(\Util\Query::GET_CONSUMER_GROUPS, $params);
            $return = array(
                'error' => false,
                'data' => $consumerGroups
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar grupos de consumo. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getConsumerGroupsByCpf($NRCPFRESPCON){
        $params = array(
            'NRCPFRESPCON' => $NRCPFRESPCON
        );
        
        try {
            $consumerGroups = $this->connection->fetchAll(\Util\Query::GET_CONSUMER_GROUPS_BY_CPF, $params);
            $return = array(
                'error' => false,
                'data'  => $consumerGroups
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar grupos de consumo. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function doCheckin($IDCHECKIN, $CDCLIENTE,$CDCONSUMIDOR){
        
        $checkedParams = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'IDCHECKIN' => $IDCHECKIN
        );
        $params = array(
            'IDCHECKIN' => $IDCHECKIN
        );

        $consumerParams = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'IDCHECKIN' =>  $IDCHECKIN
        );
        try {
            $response = $this->connection->fetchAssoc(\Util\Query::CHECK_VALID_CHECKINID, $params);
            if($response) {
                $checked = $this->connection->fetchAssoc(\Util\Query::CONSUMER_CHECKED, $checkedParams);
                if(!$checked){
                    $this->connection->executeQuery(\Util\Query::UPDATE_CHECKIN_CONSUMER, $consumerParams);
                    $response = $this->connection->fetchAssoc(\Util\Query::GET_RESULT_CHECKINMESPOS, $params);
                    $return = array(
                        'error' => false,
                        'data' => $response
                    );
                }else{
                    $return = array(
                        'error' => true,
                        'message' => 'Tentativa de checkin em duas posições diferentes'
                    );
                }   
               
            } else {
                $return = array(
                    'error' => true,
                    'message' => 'CheckinId inválido'
                );
            }   
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro Odhen\API\Service\Order @doCheckin. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getTableBill($CDFILIAL,$NRVENDAREST,$NRCOMANDA){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRVENDAREST' => $NRVENDAREST,
            'NRCOMANDA' => $NRCOMANDA
        );
        
        $errorMessage = 'Erro ao buscar a venda.';
        try {
            
            $result = $this->connection->fetchAll(\Util\Query::SQL_GET_TABLE_BILL, $params);
            
            if(count($result) > 0) {
                $return = array(
                    'error' => false,
                    'data' => $result
                );
            } else {
                $return = array(
                    'error' => true,
                    'message' => 'Nenhuma venda encontrada'
                );
            }
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => $errorMessage.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getPositionProducts($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPOSICAO) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'NRPOSICAO' => $NRPOSICAO    
        );
        
        try {
            $products = $this->connection->fetchAll(\Util\Query::SQL_GET_POSITION_PRODUCTS, $params);
            $return = array(
                'error' => false,
                'data' => $products
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos relacionados à venda '.$NRCOMANDA.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getPositionCombos($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPOSICAO) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'NRPOSICAO' => $NRPOSICAO    
        );
        
        try {
            $combos = $this->connection->fetchAll(\Util\Query::SQL_GET_POSITION_COMBOS, $params);
            $return = array(
                'error' => false,
                'data' => $combos
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar combos relacionados à venda. Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getPositionCombosProducts($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDPRODPROMOCAO, $NRSEQPRODCOM){
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'CDPRODPROMOCAO' => $CDPRODPROMOCAO,
            'NRSEQPRODCOM' => $NRSEQPRODCOM,
        );
        
        try {
            if ($this->util->databaseIsOracle()) $comboProducts = $this->connection->fetchAll(\Util\Query::ORACLE_GET_POSITION_COMBO_PRODUCTS, $params);
            else $comboProducts = $this->connection->fetchAll(\Util\Query::SQL_GET_POSITION_COMBO_PRODUCTS, $params);
            $return = array(
                'error' => false,
                'data' => $comboProducts
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar produtos dos combos relacionados à venda'.$e->getMessage()
            );
        }
        return $return;
    }

    /* public function getPositionPayments($CDFILIAL, $NRVENDAREST, $NRPOSICAO) {
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST'  => $NRVENDAREST,
            'NRCOMANDA' => $NRCOMANDA,
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        
        try {
            $payments = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PAYMENTS, $params);
                if(empty($payments) && !$isComandaVen) $payments = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PAYMENTS_MOVCAIXA, $params);
            $return = array(
                'error' => false,
                'data' => $payments
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar formas de pagamento relacionadas à venda '.$NRCOMANDA.'. Erro: '.$e->getMessage()
            );
        }
        return $return;
    } */

    /* public function getPositionProductObservations($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN){
        $params = array(
            'CDFILIAL' => $CDFILIAL, 
            'NRVENDAREST' => $NRVENDAREST, 
            'NRCOMANDA' => $NRCOMANDA,
            'NRPRODCOMVEN' => $NRPRODCOMVEN
        );
        
        try {
            if ($this->util->databaseIsOracle()) {
                $observations = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PRODUCT_OBSERVATIONS, $params);
                if(empty($observations) && !$isComandaVen) $observations = $this->connection->fetchAll(\Util\Query::GET_LASTSALE_PRODUCT_OBSERVATIONS_OBSITEMVENDA, $params);
            } else $observations = $this->connection->fetchAll(\Util\Query::GET_POSITION_PRODUCT_OBSERVATIONS, $params);
      
            $return = array(
                'error' => false,
                'data' => $observations
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro ao buscar observacoes. Erro: '.$e->getMessage()
            );
        }
        return $return;
    } */

    public function insertExtractMovement($NRSEQMOVEXT, $CDCLIENTE,  $CDCONSUMIDOR, $DSOPEEXTCONS, $VRMOVEXTCONS, $IDTPMOVEXT, $CDFAMILISALD, $CDFILIAL, $DTMOVEXTCONS, $VRSALDCONEXT){
        $params = array(
            'NRSEQMOVEXT' => $NRSEQMOVEXT,
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'DSOPEEXTCONS' => $DSOPEEXTCONS,
            'VRMOVEXTCONS' => $VRMOVEXTCONS,
            'IDTPMOVEXT' => $IDTPMOVEXT,
            'CDFAMILISALD' => $CDFAMILISALD,
            'CDFILIAL' => $CDFILIAL,
            'DTMOVEXTCONS' => $DTMOVEXTCONS,
            'VRSALDCONEXT' => $VRSALDCONEXT
        );
        $errorMessage = 'Erro ao inserir movimentacao de extrato do consumidor.';
        try {
            $result = $this->connection->executeQuery(\Util\Query::SQL_INSERT_EXTRACT_MOVEMENT, $params);
            
            $return = array(
                'error' => false,
                'message' => 'OK'
            );
        } catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => $errorMessage.'. Erro: '.$e->getMessage()
            );
        }
        return $return;   
    }

    public function getConsumerDetails($clientCode, $consumerCode){
		$params = array(
			'CDCLIENTE' => $clientCode,
			'CDCONSUMIDOR' => $consumerCode
		);
		return $this->connection->fetchAssoc(\Util\Query::GET_CONSUMER_DETAILS,$params);
	}

    public function getFamilys($cdfilial){
		$params = array(
            'CDFILIAL' => $cdfilial
		);
		return $this->connection->fetchAll(\Util\Query::GET_FAMILYS, $params);
	}

    public function getFamilyBalances($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR){
        $familys = array();

        $params = array(
            'CDFILIAL' => $CDFILIAL
        );
        $familyDetails = $this->connection->fetchAll(\Util\Query::GET_FAMILYS, $params);
        foreach($familyDetails as $family){
            $result = array();
            $CDFAMILISALD = $family['CDFAMILISALD'];

            $params = array(
                'CDCLIENTE' => $CDCLIENTE,
                'CDCONSUMIDOR' => $CDCONSUMIDOR
            );
            
            $consumoDiario = $this->localUtil->calculaConsumoDiario($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD);
            $consumoMensal = $this->localUtil->calculaConsumoMensal($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD);

            $result['familyCode'] = $CDFAMILISALD;
            $result['familyName'] = $family['NMFAMILISALD'];
            $result['sequence'] = $family['NRORDAPRSALD'];
            $result['negativeBalance'] = $family['IDSALDNEGFAM'];
            $result['spentToday'] = $consumoDiario;
            $result['spentMonthly'] = $consumoMensal;
            $result['maxDailyValue'] = !(empty($family['VRCONSDIAFAM'])) ? $this->localUtil->removeDecimal($family['VRCONSDIAFAM']) : null;

            $saldoDisponivel = $this->localUtil->buscaSaldoExtrato($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $CDFILIAL);

            if (empty($saldoDisponivel)) $result['balance'] = 0;
            else $result['balance'] = floatval($saldoDisponivel['VRSALDCONEXT']);

            array_push($familys, $result);
        }

        return $familys;
    }

    public function getDtAbercaixa($cdFilial, $cdCaixa){
        $params = array(
            'CDFILIAL' => $cdFilial,
            'CDCAIXA' => $cdCaixa
        );

        $dtAberCaixa = $this->connection->fetchAll(\Util\Query::GET_DTABERCAIX, $params);
        
        return $dtAberCaixa[0];

    }

    public function getConsumerByRef($consumerDetails){

        try {
             $params = array(
                    'CDIDCONSUMID' => $consumerDetails['consumerRef']
            );
            $consumerDetails = $this->connection->fetchAssoc(\Util\Query::GET_CONSUMER_BY_IDCONSUMER, $params);
            if(empty($consumerDetails)) {
                throw new \Exception("Não foi encontrado nenhum consumidor com o cracha informado.");
                }

            $return = array(
                'error' => false,
                'message' => '',
                'data' => $consumerDetails
            );

        }catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro: '.$e->getMessage()
            );
        }
        return $return; 
    }

    public function insertBuyCredit($rechard, $cdFilial, $cdCaixa, $dtAberCaix, $NRORG, $cdcliente, $nrsequmovi){
        $cdTipoRece = $rechard['paymentId'];
        $nsu = $rechard['nsu'];
        $vrMovVend = $rechard['value'];

        $paymentParams = array(
			'CDFILIAL' 	   	 => $cdFilial,
			'CDCAIXA' 	   	 => $cdCaixa,
			'DTABERCAIX'   	 => $dtAberCaix,
			'NRSEQUMOVI'   	 => $nrsequmovi,
			'DTHRINCMOV'	 => new \DateTime(date("d-m-Y H:i:s")),
			'IDTIPOMOVIVE' 	 => 'E',
			'VRMOVIVEND'   	 => $vrMovVend,
			'QTPARCRECEB'    => '1',
			'VRMOVIVEOUT'    => 0,
			'VRCOTINDOUT'    => 0,
			'NRSEQVENDA'   	 => null,
			'NRORG'			 => $NRORG,
			'CDTIPORECE'  	 => $cdTipoRece,
			'DTMOVIMCAIXA'	 => $dtAberCaix,
			'CDOPERINCLUSAO' => null,
			'CDCLIENTE' 	 => $cdcliente,
			'NRORGINCLUSAO'  => $NRORG,
			'CDNSUHOSTTEF'   => $nsu,
			'NRCONTROLTEF'   => null,
			'NRCARTBANCO'	 => null,
			'NRAUTORIZATEF'  => null
		);
		$type = array(
			'DTABERCAIX' => \Doctrine\DBAL\Types\Type::DATETIME,
			'DTHRINCMOV' => \Doctrine\DBAL\Types\Type::DATETIME,
			'DTMOVIMCAIXA' => \Doctrine\DBAL\Types\Type::DATE
		);

        $this->connection->executeQuery(\Util\Query::INSERT_MOVCAIXA, $paymentParams, $type);
    }

    public function saveConsumerPassword($consumerDetails, $passEncript, $passMD5){

        try {
        $params = array(
            'CDCLIENTE'      => $consumerDetails['CDCLIENTE'],
            'CDCONSUMIDOR'   => $consumerDetails['CDCONSUMIDOR'],
            'CDSENHACONS'    => $passEncript,
            'CDSENHACONSMD5' => $passMD5
        );
		
        $this->connection->executeQuery(\Util\Query::UPDATE_CONSUMER_PASS, $params);
        $return = array(
            'error' => false,
            'message' => 'Senha Cadastrada com Sucesso!'
        );

        }catch (\Exception $e) {
            $return = array(
                'error' => true,
                'message' => 'Erro: '.$e->getMessage()
            );
        }
        return $return;
    }

    public function getTypeSaleCons($CDCLIENTE, $CDCONSUMIDOR) {
        $params = array(
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        
        $data = $this->connection->fetchAssoc(\Util\Query::GET_TYPE_SALE_CONSUMER, $params);
        $type = $data['IDTPVENDACONS'];
  
        if(($type == '1') || ($type == '5')){
          $typeSale =  array(
              'allowPersonalCredit' => false ,
              'allowConsumerDebit'  => true
          );
        }elseif(($type == '2') || ($type == '6')){
          $typeSale =  array(
            'allowPersonalCredit' => true ,
            'allowConsumerDebit'  => false
        );
        }elseif(($type == '4') || ($type == '7')){
          $typeSale =  array(
            'allowPersonalCredit' => true ,
            'allowConsumerDebit'  => true
        );
        }else{
          $typeSale =  array(
            'allowPersonalCredit' => false ,
            'allowConsumerDebit'  => false
          );
        }
        $return = array(
            'error' => false,
            'data'  => $typeSale
        );
        
        return $return;
    }

    public function getConsumerbyPhone($phone) {

        $params = [
            ':NRCELULARCONS' => $phone,
            ':NRTELECONS'    => $phone,
            ':NRTELE2CONS'   => $phone,
            ':NRFAXCONS'     => $phone,
            ':CDCONSUMIDOR'  => str_pad($phone, 21, 0, STR_PAD_LEFT)
        ];

        return $this->connection->fetchAssoc(\Util\Query::GET_CONSUMER_BY_PHONE, $params);
    }

    public function getDefaultParams($cdfilial) {

        try {
            $params = [
                ':CDFILIAL' => $cdfilial
            ];

            $result = $this->connection->fetchAssoc(\Util\Query::GET_CONSUMER_DEFAULT_PARAMS, $params);

            if(empty($result['CDCLIENTE'])) {
                throw new \Exception("Cliente padrão não cadastrado.");
            }

            if(empty($result['CDCCUSCLIE'])) {
                throw new \Exception("Centro de Custo padrão não cadastrado.");
            }

            if(empty($result['CDTIPOCONS'])) {
                throw new \Exception("Tipo de consumidor padrão não cadastrado.");
            }

            $return = [
                'error'   => false,
                'message' => '',
                'data'    => $result
            ];
        } catch (\Exception $e) {
            $return = [
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ];
        }

        return $return;
    }
}